@@ -1,5 +1,6 @@
 import re
 
+from time import perf_counter
 from typing import Any
 from typing import Dict
 from typing import Optional
@@ -7,7 +8,9 @@ from typing import Sequence
 from typing import Tuple
 from typing import Union
 
+from prometheus_client import Counter
 from prometheus_client import Gauge
+from prometheus_client import Histogram
 from sqlalchemy import create_engine
 from sqlalchemy import event
 from sqlalchemy.engine import Connection
@@ -24,6 +27,7 @@ from baseplate import SpanObserver
 from baseplate.clients import ContextFactory
 from baseplate.lib import config
 from baseplate.lib import metrics
+from baseplate.lib.prometheus_metrics import default_latency_buckets
 from baseplate.lib.secrets import SecretsStore
 
 
@@ -156,51 +160,69 @@ class SQLAlchemyEngineContextFactory(ContextFactory):
 
     """
 
-    PROM_PREFIX = "bp_sqlalchemy_pool"
-    PROM_LABELS = ["pool"]
+    PROM_PREFIX = "sql_client"
+    PROM_POOL_PREFIX = f"{PROM_PREFIX}_pool"
+    PROM_POOL_LABELS = ["sql_pool"]
 
     max_connections_gauge = Gauge(
-        f"{PROM_PREFIX}_max_size",
+        f"{PROM_POOL_PREFIX}_max_size",
         "Maximum number of connections allowed in this pool",
-        PROM_LABELS,
-    )
-
-    checked_in_connections_gauge = Gauge(
-        f"{PROM_PREFIX}_idle_connections",
-        "Number of available, checked in, connections in this pool",
-        PROM_LABELS,
+        PROM_POOL_LABELS,
     )
 
     checked_out_connections_gauge = Gauge(
-        f"{PROM_PREFIX}_active_connections",
-        "Number of connections in use, or checked out, in this pool",
+        f"{PROM_POOL_PREFIX}_active_connections",
+        "Number of connections in use by this pool (checked out + overflow)",
+        PROM_POOL_LABELS,
+    )
+    checked_in_connections_gauge = Gauge(
+        f"{PROM_POOL_PREFIX}_idle_connections",
+        "Number of connections not in use by this pool (unused pool connections)",
+        PROM_POOL_LABELS,
+    )
+
+    PROM_LABELS = [
+        "sql_pool",
+        "sql_address",
+        "sql_database",
+    ]
+
+    latency_seconds = Histogram(
+        f"{PROM_PREFIX}_latency_seconds",
+        "Latency histogram of calls to database",
+        PROM_LABELS + ["sql_success"],
+        buckets=default_latency_buckets,
+    )
+
+    active_requests = Gauge(
+        f"{PROM_PREFIX}_active_requests",
+        "total requests that are in-flight",
         PROM_LABELS,
     )
 
-    overflow_connections_gauge = Gauge(
-        f"{PROM_PREFIX}_overflow_connections",
-        "Number of connections over the desired size of this pool",
-        PROM_LABELS,
+    requests_total = Counter(
+        f"{PROM_PREFIX}_requests_total",
+        "Total number of sql requests",
+        PROM_LABELS + ["sql_success"],
     )
 
     def __init__(self, engine: Engine, name: str = "sqlalchemy"):
         self.engine = engine.execution_options()
+        self.name = name
         event.listen(self.engine, "before_cursor_execute", self.on_before_execute, retval=True)
         event.listen(self.engine, "after_cursor_execute", self.on_after_execute)
         event.listen(self.engine, "handle_error", self.on_error)
-
-        pool = self.engine.pool
-        if isinstance(pool, QueuePool):
-            self.max_connections_gauge.labels(name).set_function(pool.size)
-            self.checked_in_connections_gauge.labels(name).set_function(pool.checkedin)
-            self.checked_out_connections_gauge.labels(name).set_function(pool.checkedout)
-            self.overflow_connections_gauge.labels(name).set_function(pool.overflow)
+        self.time_started = 0.0
 
     def report_runtime_metrics(self, batch: metrics.Client) -> None:
         pool = self.engine.pool
         if not isinstance(pool, QueuePool):
             return
 
+        self.max_connections_gauge.labels(self.name).set(pool.size())
+        self.checked_out_connections_gauge.labels(self.name).set(pool.checkedout())
+        self.checked_in_connections_gauge.labels(self.name).set(pool.checkedin())
+
         batch.gauge("pool.size").replace(pool.size())
         batch.gauge("pool.open_and_available").replace(pool.checkedin())
         batch.gauge("pool.in_use").replace(pool.checkedout())
@@ -221,6 +243,14 @@ class SQLAlchemyEngineContextFactory(ContextFactory):
         executemany: bool,
     ) -> Tuple[str, Parameters]:
         """Handle the engine's before_cursor_execute event."""
+        labels = {
+            "sql_pool": self.name,
+            "sql_address": conn.engine.url.host,
+            "sql_database": conn.engine.url.database,
+        }
+        self.active_requests.labels(**labels).inc()
+        self.time_started = perf_counter()
+
         context_name = conn._execution_options["context_name"]
         server_span = conn._execution_options["server_span"]
 
@@ -254,12 +284,36 @@ class SQLAlchemyEngineContextFactory(ContextFactory):
         conn.info["span"].finish()
         conn.info["span"] = None
 
+        labels = {
+            "sql_pool": self.name,
+            "sql_address": conn.engine.url.host,
+            "sql_database": conn.engine.url.database,
+        }
+
+        self.active_requests.labels(**labels).dec()
+        self.requests_total.labels(**labels, sql_success="true").inc()
+        self.latency_seconds.labels(**labels, sql_success="true").observe(
+            perf_counter() - self.time_started
+        )
+
     def on_error(self, context: ExceptionContext) -> None:
         """Handle the event which happens on exceptions during execution."""
         exc_info = (type(context.original_exception), context.original_exception, None)
         context.connection.info["span"].finish(exc_info=exc_info)
         context.connection.info["span"] = None
 
+        labels = {
+            "sql_pool": self.name,
+            "sql_address": context.connection.engine.url.host,
+            "sql_database": context.connection.engine.url.database,
+        }
+
+        self.active_requests.labels(**labels).dec()
+        self.requests_total.labels(**labels, sql_success="false").inc()
+        self.latency_seconds.labels(**labels, sql_success="false").observe(
+            perf_counter() - self.time_started
+        )
+
 
 class SQLAlchemySessionContextFactory(SQLAlchemyEngineContextFactory):
     """SQLAlchemy ORM session context factory.
