@@ -1,13 +1,12 @@
 """Helpers for interacting with ZooKeeper."""
 from typing import Optional
 
-import gevent
-
 from kazoo.client import KazooClient
 from kazoo.handlers.gevent import SequentialGeventHandler
 
 from baseplate.lib import config
 from baseplate.lib.secrets import SecretsStore
+from baseplate.server.monkey import gevent_is_patched
 
 
 def zookeeper_client_from_config(
@@ -56,12 +55,18 @@ def zookeeper_client_from_config(
         credentials = secrets.get_simple(path)
         auth_data.append(("digest", credentials.decode("utf8")))
 
+    # Kazoo needs to use a different handler with gevent.
+    if gevent_is_patched():
+        handler = SequentialGeventHandler()
+    else:
+        handler = None
+
     return KazooClient(
         cfg.hosts,
         timeout=cfg.timeout.total_seconds(),
         auth_data=auth_data,
         read_only=read_only,
-        handler=SequentialGeventHandler(),
+        handler=handler,
         # this retry policy tells Kazoo how often it should attempt connections
         # to ZooKeeper from its worker thread/greenlet. when the connection is
         # lost during normal operation (i.e. after it was first established)
@@ -83,6 +88,5 @@ def zookeeper_client_from_config(
             backoff=2,  # exponential backoff
             max_jitter=1,  # maximum amount to jitter sleeptimes
             max_delay=60,  # never wait longer than this
-            sleep_func=gevent.sleep,
         ),
     )
