@@ -8,8 +8,6 @@ from unittest import mock
 import gevent.monkey
 import pytest
 
-from thrift.Thrift import TApplicationException
-
 from baseplate import Baseplate
 from baseplate import BaseplateObserver
 from baseplate import ServerSpanObserver
@@ -19,8 +17,6 @@ from baseplate.clients.thrift import ThriftClient
 from baseplate.frameworks.thrift import baseplateify_processor
 from baseplate.lib import config
 from baseplate.lib.thrift_pool import ThriftConnectionPool
-from baseplate.observers.prometheus import PrometheusClientSpanObserver
-from baseplate.observers.prometheus import PrometheusServerSpanObserver
 from baseplate.observers.timeout import ServerTimeout
 from baseplate.observers.timeout import TimeoutBaseplateObserver
 from baseplate.server import make_listener
@@ -634,283 +630,3 @@ class ThriftPrometheusMetricsTests(GeventPatchedTestCase):
         self.assertEqual(got_labels, want_labels)
         got_value = sample[2]
         self.assertEqual(got_value, want_value)
-
-    def test_prom_observer_metrics_success(self):
-        """The PrometheusServerSpanObserver should set labels and metrics correctly for successful requests."""
-
-        class Handler(TestService.Iface):
-            def example(self, context):
-                return True
-
-        prom_observer = PrometheusServerSpanObserver()
-        with serve_thrift(Handler(), TestService, prom_observer) as server:
-            with raw_thrift_client(server.endpoint, TestService) as client:
-                client.example()
-
-        prom_labels = prom_observer.get_labels()
-        self.assertEqual(
-            prom_labels,
-            {
-                "protocol": "thrift",
-                "success": "true",
-                "thrift.method": "example",
-            },
-        )
-
-        m = prom_observer.metrics
-        self.assert_correct_metric(
-            m.get_active_requests_metric(),
-            1,
-            0,
-            "thrift_server_active_requests",
-            {"thrift_method": "example"},
-            0.0,
-        )
-        self.assert_correct_metric(
-            m.get_latency_seconds_metric(),
-            18,
-            3,
-            "thrift_server_latency_seconds_bucket",
-            {"thrift_method": "example", "thrift_success": "true", "le": "0.0015625"},
-            1.0,
-        )
-        self.assert_correct_metric(
-            m.get_requests_total_metric(),
-            2,
-            0,
-            "thrift_server_requests_total",
-            {
-                "thrift_baseplate_status": "",
-                "thrift_baseplate_status_code": "",
-                "thrift_exception_type": "",
-                "thrift_method": "example",
-                "thrift_success": "true",
-            },
-            1.0,
-        )
-        self.reset_metrics(prom_observer.metrics)
-
-    def test_prom_observer_metrics_app_exc(self):
-        """The PrometheusServerSpanObserver should set labels and metrics correctly for requests that throw an exception defined by the service."""
-
-        class Handler(TestService.Iface):
-            def example(self, context):
-                raise TestService.ExpectedException()
-
-        prom_observer = PrometheusServerSpanObserver()
-        with serve_thrift(Handler(), TestService, prom_observer) as server:
-            with raw_thrift_client(server.endpoint, TestService) as client:
-                with self.assertRaises(TestService.ExpectedException):
-                    client.example()
-
-        got_labels = prom_observer.get_labels()
-        want_labels = {
-            "protocol": "thrift",
-            "thrift.method": "example",
-            "exception_type": "ExpectedException",
-            "success": "false",
-        }
-        self.assertEqual(got_labels, want_labels)
-
-        m = prom_observer.metrics
-        self.assert_correct_metric(
-            m.get_active_requests_metric(),
-            1,
-            0,
-            "thrift_server_active_requests",
-            {"thrift_method": "example"},
-            0.0,
-        )
-        self.assert_correct_metric(
-            m.get_latency_seconds_metric(),
-            18,
-            3,
-            "thrift_server_latency_seconds_bucket",
-            {"thrift_method": "example", "thrift_success": "false", "le": "0.0015625"},
-            1.0,
-        )
-        self.assert_correct_metric(
-            m.get_requests_total_metric(),
-            2,
-            0,
-            "thrift_server_requests_total",
-            {
-                "thrift_baseplate_status": "",
-                "thrift_baseplate_status_code": "",
-                "thrift_exception_type": "ExpectedException",
-                "thrift_method": "example",
-                "thrift_success": "false",
-            },
-            1.0,
-        )
-        self.reset_metrics(prom_observer.metrics)
-
-    def test_prom_observer_metrics_thrift_exc(self):
-        """The PrometheusServerSpanObserver should set labels and metrics correctly for requests that throw a thrift exception."""
-
-        class Handler(TestService.Iface):
-            def example(self, context):
-                raise TApplicationException
-
-        prom_observer = PrometheusServerSpanObserver()
-        with serve_thrift(Handler(), TestService, prom_observer) as server:
-            with raw_thrift_client(server.endpoint, TestService) as client:
-                with self.assertRaises(TApplicationException):
-                    client.example()
-
-        got_labels = prom_observer.get_labels()
-        want_labels = {
-            "protocol": "thrift",
-            "thrift.method": "example",
-            "exception_type": "TApplicationException",
-            "success": "false",
-        }
-        self.assertEqual(got_labels, want_labels)
-
-        m = prom_observer.metrics
-        self.assert_correct_metric(
-            m.get_active_requests_metric(),
-            1,
-            0,
-            "thrift_server_active_requests",
-            {"thrift_method": "example"},
-            0.0,
-        )
-        self.assert_correct_metric(
-            m.get_latency_seconds_metric(),
-            18,
-            3,
-            "thrift_server_latency_seconds_bucket",
-            {"thrift_method": "example", "thrift_success": "false", "le": "0.0015625"},
-            1.0,
-        )
-        self.assert_correct_metric(
-            m.get_requests_total_metric(),
-            2,
-            0,
-            "thrift_server_requests_total",
-            {
-                "thrift_baseplate_status": "",
-                "thrift_baseplate_status_code": "",
-                "thrift_exception_type": "TApplicationException",
-                "thrift_method": "example",
-                "thrift_success": "false",
-            },
-            1.0,
-        )
-        self.reset_metrics(prom_observer.metrics)
-
-    def test_prom_observer_metrics_baseplate_exc(self):
-        """
-        If the server returns an baseplate thrift Error, then the = thrift.status_code label and
-        thrift.status label should be set.
-        """
-
-        class Handler(TestService.Iface):
-            def example(self, context):
-                raise Error(ErrorCode.NOT_FOUND, "foo is not found")
-
-        prom_observer = PrometheusServerSpanObserver()
-        with serve_thrift(Handler(), TestService, prom_observer) as server:
-            with raw_thrift_client(server.endpoint, TestService) as client:
-                with self.assertRaises(Error):
-                    client.example()
-
-        got_labels = prom_observer.get_labels()
-        want_labels = {
-            "protocol": "thrift",
-            "thrift.method": "example",
-            "exception_type": "Error",
-            "thrift.status_code": 404,
-            "thrift.status": "NOT_FOUND",
-            "success": "false",
-        }
-        self.assertEqual(got_labels, want_labels)
-
-        m = prom_observer.metrics
-        self.assert_correct_metric(
-            m.get_active_requests_metric(),
-            1,
-            0,
-            "thrift_server_active_requests",
-            {"thrift_method": "example"},
-            0.0,
-        )
-        self.assert_correct_metric(
-            m.get_latency_seconds_metric(),
-            18,
-            3,
-            "thrift_server_latency_seconds_bucket",
-            {"thrift_method": "example", "thrift_success": "false", "le": "0.0015625"},
-            1.0,
-        )
-        self.assert_correct_metric(
-            m.get_requests_total_metric(),
-            2,
-            0,
-            "thrift_server_requests_total",
-            {
-                "thrift_baseplate_status": "NOT_FOUND",
-                "thrift_baseplate_status_code": "404",
-                "thrift_exception_type": "Error",
-                "thrift_method": "example",
-                "thrift_success": "false",
-            },
-            1.0,
-        )
-        self.reset_metrics(prom_observer.metrics)
-
-    def test_client_metrics(self):
-        class Handler(TestService.Iface):
-            def __init__(self):
-                self.edge_context = None
-
-            def example(self, context):
-                self.edge_context = context.edge_context
-                return True
-
-        handler = Handler()
-
-        prom_observer = PrometheusClientSpanObserver()
-        # This normally called when a child span is created in
-        # PrometheusServerSpanObserver, we're skipping that
-        prom_observer.on_set_tag("protocol", "thrift")
-
-        with serve_thrift(handler, TestService) as server:
-            with baseplate_thrift_client(server.endpoint, TestService, prom_observer) as context:
-                context.example_service.example()
-
-        self.assert_correct_metric(
-            prom_observer.metrics.get_active_requests_metric(),
-            1,
-            0,
-            "thrift_client_active_requests",
-            {
-                "thrift_slug": "example_service",
-                "thrift_method": "example",
-            },
-            0.0,
-        )
-
-        self.assert_correct_metric(
-            prom_observer.metrics.get_requests_total_metric(),
-            2,
-            0,
-            "thrift_client_requests_total",
-            {
-                "thrift_slug": "example_service",
-                "thrift_success": "true",
-                "thrift_exception_type": "",
-                "thrift_baseplate_status": "",
-                "thrift_baseplate_status_code": "",
-            },
-            1.0,
-        )
-        self.assert_correct_metric(
-            prom_observer.metrics.get_latency_seconds_metric(),
-            18,
-            14,
-            "thrift_client_latency_seconds_bucket",
-            {"thrift_slug": "example_service", "thrift_success": "true", "le": "+Inf"},
-            1.0,
-        )
