@@ -10,11 +10,89 @@ except ImportError:
 else:
     del pymemcache
 
+from prometheus_client import REGISTRY
 from baseplate.lib.config import ConfigurationError
-from baseplate.clients.memcache import pool_from_config, MemcacheContextFactory
+from baseplate.clients.memcache import pool_from_config
+from baseplate.clients.memcache import MonitoredMemcacheConnection
 from baseplate.clients.memcache import lib as memcache_lib
 
 
+class PrometheusInstrumentationTests(unittest.TestCase):
+    def setUp(self):
+        # stub out the server span, each call to MonitoredMemcacheConnection
+        # will create a new span - but we don't use that for prom
+        span_inner = mock.Mock()
+        span_inner.__enter__ = mock.Mock(return_value=mock.Mock())
+        span_inner.__exit__ = mock.Mock(return_value=None)
+        self.span = mock.Mock()
+        self.span.make_child = lambda trace_name: span_inner
+
+    # test the @_prom_instrument decorator by calling one of the decorated functions
+    def test_prometheus_decorator(self):
+        # stub out the client, we only use the server name
+        pooled_client = mock.Mock()
+        pooled_client.server = "server name"
+
+        prom_labels = {
+            "memcached_address": "server name",
+            "memcached_command": "incr",
+        }
+
+        mmc = MonitoredMemcacheConnection("test", self.span, pooled_client)
+        mmc.incr("blah", 1)
+
+        assert (
+            REGISTRY.get_sample_value(
+                "memcached_client_active_requests",
+                prom_labels,
+            )
+            == 0
+        )
+        assert (
+            REGISTRY.get_sample_value(
+                "memcached_client_requests_total",
+                {
+                    **prom_labels,
+                    "memcached_success": "true",
+                },
+            )
+            == 1
+        )
+        assert (
+            REGISTRY.get_sample_value(
+                "memcached_client_latency_seconds_bucket",
+                {
+                    **prom_labels,
+                    "memcached_success": "true",
+                    "le": "+Inf",
+                },
+            )
+            == 1
+        )
+
+    def test_prometheus_decorator_failing(self):
+        class FailClient(mock.Mock):
+            server = "hello"
+
+            def incr(self, _a, _b, noreply=None):
+                raise TypeError
+
+        fail_mmc = MonitoredMemcacheConnection("test", self.span, FailClient())
+        self.assertRaises(TypeError, fail_mmc.incr, "aaa", 1)
+
+        assert (
+            REGISTRY.get_sample_value(
+                "memcached_client_requests_total",
+                {
+                    "memcached_address": "hello",
+                    "memcached_command": "incr",
+                    "memcached_success": "false",
+                },
+            )
+            == 1
+        )
+
+
 class PoolFromConfigTests(unittest.TestCase):
     def test_empty_config(self):
         with self.assertRaises(ConfigurationError):
@@ -54,17 +132,6 @@ class PoolFromConfigTests(unittest.TestCase):
         )
         self.assertEqual(pool.no_delay, False)
 
-    def test_metrics(self):
-        max_pool_size = "123"
-        ctx = MemcacheContextFactory(
-            pool_from_config(
-                {"memcache.endpoint": "localhost:1234", "memcache.max_pool_size": max_pool_size}
-            )
-        )
-        metric = ctx.pool_size_gauge.collect()[0]
-        sample = [sample for sample in metric.samples if sample.labels["pool"] == "default"][0]
-        self.assertEqual(sample.value, float(max_pool_size))
-
 
 class SerdeTests(unittest.TestCase):
     def test_serialize_str(self):
